using System;
using System.Collections.Generic;
using System.Text;

using System.Data;
using System.Data.SqlClient;
using OOP.Bank.Common;
using OOP.Bank.Interface;
using OOP.Bank.DataAccess;

namespace OOP.Bank.Business
{
    public class Account : IBankAccount
    {
        private String _account_no;
        private Double _account_balance;
        private ITransaction _transaction;
         

        public Account()
        {
        }

        public Account(String AccountNo)
        {
            SqlDataReader account;
            account = AccountDac.Retrieve(AccountNo);

            if (account.Read())
            {
                _account_no = AccountNo;
                _account_balance = Convert.ToDouble(account["_account_balance"]);
                Transaction transaction = new Transaction(AccountNo);
                _transaction = (ITransaction)Transaction;
            }
            account.Close();
        }

        public String AccountNo
        {
            get { return _account_no; }
            set { _account_no = value; }
        }

        public Double AccountBalance
        {
            get { return _account_balance; }
            set { _account_balance = value; }
        }

        public ITransaction Transaction
        {
            get { return _transaction; }
            set { _transaction = value; }
        }

        public Double MaintainingBalance
        {
            get { return Configuration.MaintainingBalance; }
        }

        
        public Int32 Create()
        {
            return AccountDac.Create(this);
        }

        public Int32 Deposit(Double Amount)
        {
            try
            {
                this.AccountBalance += Amount;
                AccountDac.Update(this);

                Transaction transaction = new Transaction();
                transaction.AccountNo = this.AccountNo;
                transaction.Operation = "Deposit";
                transaction.Amount = Amount;
                transaction.Create();

                return 1;
            }
            catch
            {
                return 0;
            }
        }

        public Int32 Deposit(String AccountNo, Double Amount)
        {
            try
            {
                this.AccountBalance += Amount;
                AccountDac.Update(this);

                Transaction transaction = new Transaction();
                transaction.AccountNo = this.AccountNo;
                transaction.Operation = "Received funds from " + AccountNo;
                transaction.Amount = Amount;
                transaction.Create();

                return 1;
            }
            catch
            {
                return 0;
            }
        }

        public Int32 Withdraw(Double Amount)
        {
            if (this.MaintainingBalance <= (this.AccountBalance - Amount))
            {
                try
                {
                    this.AccountBalance -= Amount;
                    AccountDac.Update(this);

                    Transaction transaction = new Transaction();
                    transaction.AccountNo = this.AccountNo;
                    transaction.Operation = "Withdraw";
                    transaction.Amount = Amount;
                    transaction.Create();

                    return 1;
                }
                catch
                {
                    return 0;
                }
            }
            else
            {
                return 2;
            }
        }

        public Int32 Withdraw(String AccountNo, Double Amount)
        {
            if (this.MaintainingBalance <= (this.AccountBalance - Amount))
            {
                try
                {
                    this.AccountBalance -= Amount;
                    AccountDac.Update(this);

                    Transaction transaction = new Transaction();
                    transaction.AccountNo = this.AccountNo;
                    transaction.Operation = "Transfered funds to " + AccountNo;
                    transaction.Amount = Amount;
                    transaction.Create();

                    return 1;
                }
                catch
                {
                    return 0;
                }
            }
            else
            {
                return 2;
            }
        }

        public Int32 Transfer(String Recipient, Double Amount)
        {
            Int32 result = this.Withdraw(Recipient, Amount);
            if (result == 1)
            {
                String Depositor = this.AccountNo;
                Account recipientAccount = new Account(Recipient);
                recipientAccount.Deposit(Depositor, Amount);
                return 1;
            }
            else
            {
                return result;
            }
        }

        public DataSet RetrieveList()
        {
            return AccountDac.RetrieveList(this);
        }
    }
}
